﻿-- Initialization
AltManager = LibStub("AceAddon-3.0"):NewAddon("AltManager", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub('AceLocale-3.0'):GetLocale('AltManager')
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)
local me = GetUnitName("Player").." - "..GetRealmName()

local listChars = { }

-- Main thing
local DBDefault, skills, tasks, questsList, taskList, global

function AltManager:PopulateVars(DBDefault_, skills_, tasks_, questsList_, taskList_, global_)
	DBDefault = DBDefault_
	skills = skills_
	tasks = tasks_
	questsList = questsList_
	taskList = taskList_
	global = global_
end

-- Let's go
function AltManager:OnInitialize()
	-- Load SV // Register slash
	self:RegisterChatCommand("am","SlashHandler")
	self:RegisterChatCommand("AltManager","SlashHandler")
	self.db = LibStub("AceDB-3.0"):New("AMdb", DBDefault)
end

function AltManager:OnEnable()
	-- Register events // hook funct // frames // init funct
	self:GetWeeklyReset()
	self:CreateMinimapFrame()
	self:RegisterEvent("QUEST_LOG_UPDATE", "QuestUpdateHandle") -- Look for quests to track
	self:RegisterEvent("PLAYER_LOGOUT", "RequestInstanceInfo") -- Hook raid ID if the player quit inside the instance
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "RequestInstanceInfo") -- Going outside of an instance where you're now tagged
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "CheckBG") -- Battleground end
	self:RegisterEvent("UPDATE_INSTANCE_INFO", "UpdateInstanceInfo") -- After RequestRaidInfo()
	self:RegisterEvent("LFG_COMPLETION_REWARD", "RequestLFGInfo") -- Fire at end of random
	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO", "CheckFrost") -- Fire after updating the rewards
	self:RegisterEvent("PLAYER_LEVEL_UP", "CheckLevel")
	self:RegisterEvent("CHAT_MSG_SKILL", "GetSkillsRank")
	self:RegisterEvent("SKILL_LINES_CHANGED", "SkillRanksReady") -- Skills ranks aren't ready at PLAYER_LOGIN, need to wait
end

function AltManager:SkillRanksReady()
	self:UnregisterEvent("SKILL_LINES_CHANGED")
	self:GetSkillsRank()
	self:LoadSV()
	self:CheckIDs()
	self:CheckBG()
	self:CheckFrost()
	self:ScheduleTimer("LoadSV",GetQuestResetTime()+5) -- Refresh after dailies quests reset
end

--- FUNCTIONS
function AltManager:SlashHandler(msg)
	if msg == "hide" then
		self.db.profile.minimap.hide = not self.db.profile.minimap.hide
		if self.db.profile.minimap.hide then
			LDBIcon:Hide("AltManager")
			self:Printf(L.SlashCmdHidden)
		else
			LDBIcon:Show("AltManager")
			self:Printf(L.SlashCmdShown)
		end
	elseif msg == "list" then
		local i = 1
		for k,v in pairs(self.db.global) do
			listChars[i] = k
			self:Printf("%s - %s",i,k)
			i = i + 1
		end
	elseif string.find(msg,"del") then
		local num = tonumber(string.match(msg,"%d+"))
		if num and listChars[num] then
			self:Printf(L.SlashCmdDeleting,num,listChars[num])
			self.db.global[listChars[num]] = nil
		elseif num then
			self:Printf(L.SlashCmdCharNExist,num)
		end
	else
		self:Printf(L.SlashCmdHelp)
		self:Printf("/am list :: %s",L.SlashCmdList)
		self:Printf("/am del # :: %s",L.SlashCmdDelete)
		self:Printf("/am hide :: %s",L.SlashCmdHide)
	end
end

-- Callbacks
function AltManager:Quest_Abandoned(uid)
	if questsList[uid] then
		self:SetDone(me, questsList[uid], 0)
	end
end

function AltManager:Quest_Gained(uid)
	if questsList[uid] then
		self:SetDone(me, questsList[uid], 1)
	end
end

function AltManager:Quest_Completed(uid)
	if questsList[uid] then
		if (self:GetStatus(me, questsList[uid]) == 1) then
			-- if questsList[uid] ~= "Weekly" then 
				self:SetDone(me, questsList[uid], 2, GetQuestResetTime())
			-- else
				-- self:SetDone(me, questsList[uid], 2, self.db.global[me].Char.reset)
			-- end
		end
	end
end

-- Public methods of the tasks
function AltManager:SetDone(who, task, number, reset)
	self.db.global[who][task].done = number
	if reset then -- if reset param
		if reset < time() then -- adaptative reset timer set
			self.db.global[who][task].reset = time() + reset
		else
			self.db.global[who][task].reset = reset
		end
	end
end

function AltManager:GetStatus(who, task)
	if self.db.global[who][task] and self.db.global[who][task].done ~= nil then
		return self.db.global[who][task].done
	else
		return DBDefault.global[me][task].done
	end
end
 
function AltManager:SetCapableOf(who, task)
	if self.db.global[who][task] then
		self.db.global[who][task].capable = true
	end
end

function AltManager:IsCapableOf(who, task)
	if self.db.global[who][task] and self.db.global[who][task].capable ~= nil then
		return self.db.global[who][task].capable
	else
		return DBDefault.global[me][task].capable
	end
end

function AltManager:Handle(who, task)
	if self.db.global[who][task] and self.db.global[who][task].handle ~= nil then
		return self.db.global[who][task].handle
	else
		return DBDefault.global[me][task].handle
	end
end

function AltManager:AddPersonalQuest(uid)
	if not self.db.global[me].pQuest then self.db.global[me].pQuest = {} end
	if not self.db.global[me].pQuest[uid] then
		local qName = self:ScanQuestLog()[uid]
		tasks[uid] = { div = "pQuest", isDaily = true, levelRequire = 1, rankRequire = nil }
		questsList[uid] = uid
		L[uid] = qName
		if not taskList.pQuest then taskList.pQuest = {} end
		tinsert(taskList.pQuest, uid)
		if not self.db.global[me].pQuest then self.db.global[me].pQuest = {} end
		self.db.global[me].pQuest[uid] = qName
		self.db.global[me][uid] = { capable = true, done = 1, handle = true, }
	end
end

function AltManager:SetPersonalQuest(uid, qName)
	if not questsList[uid] then
		tasks[uid] = { div = "pQuest", isDaily = true, levelRequire = 1, rankRequire = nil }
		questsList[uid] = uid
		L[uid] = qName
		if not taskList.pQuest then taskList.pQuest = {} end
		tinsert(taskList.pQuest, uid)
	end
end

function AltManager:GetPersonalQuests()
	if (self.db.global[me].pQuest) then
		return self.db.global[me].pQuest
	end
	return {}
end

function AltManager:DeletePersonalQuest(uid)
	if (self.db.global[me].pQuest and self.db.global[me].pQuest[uid]) then
		self.db.global[me].pQuest[uid] = nil
	end
	if (self.db.global[me][uid]) then
		self.db.global[me][uid] = nil
	end
	if (taskList.pQuest and taskList.pQuest[uid]) then
		taskList.pQuest[uid] = nil
	end
end

function AltManager:IsQuestTrackable(uid)
	if not questsList[uid] then return true end
	return false
end

-- Lots of checking functions
function AltManager:CheckLevel()
	local lvl = UnitLevel("player")
	for k,_ in pairs(tasks) do
		if lvl >= tasks[k].levelRequire then
			if not tasks[k].rankRequire or skills.Rank[k] >= tasks[k].rankRequire then
				self:SetCapableOf(me,k)
			end
		end
	end
end

-- Look for the biggest weekly reset in raid saved instances
function AltManager:RequestInstanceInfo()
	RequestRaidInfo()
	self.db.global[me].Char.level = UnitLevel("Player")
end

function AltManager:UpdateInstanceInfo()
	self:GetWeeklyReset()
	self:CheckIDs()
end

function AltManager:GetWeeklyReset()
	local biggestResetTime = 0
	for i = 1, GetNumSavedInstances() do
		local _, _, reset, _, locked, extended, _, isRaid = GetSavedInstanceInfo(i)
		if isRaid and (locked or extended) then
			if reset > biggestResetTime then biggestResetTime = reset end
		end
	end
	self.db.global[me].Char.reset = biggestResetTime
end

function AltManager:CheckIDs()
	local sInstances = { }
	for i = 1, GetNumSavedInstances() do
		local name, _, reset, difficulty, locked, extended, _, isRaid, maxPlayers, _, numEncounters, encounterProgress  = GetSavedInstanceInfo(i)
		if isRaid then
			sInstances[name] = { ["reset"] = reset, ["diff"] = difficulty, ["lock"] = locked, ["ext"] = extended, ["maxP"] = maxPlayers, ["numE"] = numEncounters, ["encounterP"] = encounterProgress }
		end
	end
	
	for key,k in pairs(taskList.raids) do
		if tasks[k].div == "raid" then
			if sInstances[L[k]] then
				if sInstances[L[k]].lock or sInstances[L[k]].ext then 
					if sInstances[L[k]].encounterP < sInstances[L[k]].numE then 
						self:SetDone(me, k, 1, sInstances[L[k]].reset) 
					else
						self:SetDone(me, k, 2, sInstances[L[k]].reset) 
					end
				else 
					self:SetDone(me, k, 0) 
				end
			else
				self:SetDone(me, k, 0) 
			end
		end
	end
end

function AltManager:RequestLFGInfo()
	RequestLFDPlayerLockInfo()
end

function AltManager:CheckFrost()
	if self:IsCapableOf(me, "Frost") then
		if UnitLevel("player") == global.lvlMax then
			if GetLFGDungeonRewards(global.InstanceID.heroic) then self:SetDone(me, "Frost", 2, GetQuestResetTime()) else self:SetDone(me, "Frost", 0) end
		else
			if GetLFGDungeonRewards(global.InstanceID.normal) then self:SetDone(me, "Frost", 2, GetQuestResetTime()) else self:SetDone(me, "Frost", 0) end
		end
	end
end

function AltManager:CheckBG()
	if self:IsCapableOf(me, "BG") then
		if (GetHolidayBGHonorCurrencyBonuses() or GetRandomBGHonorCurrencyBonuses()) then self:SetDone(me, "BG", 2, GetQuestResetTime()) else self:SetDone(me, "BG", 0) end
	end
end

function AltManager:LocalizeSkillsName()
	skills.Name = {}
	for k, v in pairs(skills.ID) do
		skills.Name[k] = GetSpellInfo(v)
	end
end

function AltManager:GetSkillsRank()
	if not skills.Name then self:LocalizeSkillsName() end
	local prof = {GetProfessions()}
	for k,v in pairs(prof) do
		local skillName, _, skillRankInfo, skillMax = GetProfessionInfo(v)
		for key, value in pairs(skills.Name) do
			if value == skillName then skills.Rank[key] = skillRankInfo end
		end
	end
	self:CheckLevel()
end

-- Loading saved variables and checking for dailies/weekly expiry
function AltManager:LoadSV()
	if (self.db.global) then
		if (self.db.global[me].pQuest) then
			for kQ, vQ in pairs(self.db.global[me].pQuest) do
				self:SetPersonalQuest(kQ,vQ)
			end
		end
		for k1, v1 in pairs(self.db.global) do -- k1 == list of chars
			for k2, v2 in pairs(v1) do -- k2 == list of tasks
				local dbDone, dbReset
				for k3, v3 in pairs(v2) do -- k3 == status
					if k3 == "done" then dbDone = v3
					elseif k3 == "reset" then dbReset = v3
					end
				end
				
				if (type(tonumber(k2)) == "number") then 
					tasks[k2] = { div = "pQuest", isDaily = true, levelRequire = 1, rankRequire = nil }
				end
				
				if dbDone then
					if not dbReset then
						if not tasks[k2] or tasks[k2].isDaily then dbReset = GetQuestResetTime() else dbReset = self.db.global[k1].Char.reset end
					end
					if dbDone == 2 then
						if dbReset < time() then self:SetDone(k1, k2, 0) else self:SetDone(k1, k2, 2, dbReset) end
					elseif dbDone == 1 and tasks[k2].div == "raid" then
						if dbReset < time() then self:SetDone(k1, k2, 0) else self:SetDone(k1, k2, 1, dbReset) end
					end
				end
			end
		end
	end
end

-- Minimap tooltip functions
function AltManager:ttformat(text,done)
	if text then
		if done == 0 then
			return "|cFFFF0000"..text.."|r"
		elseif done == 1 then
			return text
		elseif done == 2 then
			return "|cFF00FF00"..text.."|r"
		end
	end
	return ""
end

function AltManager:CreateMinimapFrame()
	if LDB then
		local numPersoSelectionne
		local alt = me
		local listePerso = { }
		local i = 1
		if (self.db.global) then
			for k,v in pairs(self.db.global) do
				if self:AnythingOnTooltip(k) then
					listePerso[i] = k
					if k == me then numPersoSelectionne = i end
					i = i + 1
				end
			end
		end
		if numPersoSelectionne == nil then 
			--if tC(listePerso) == 0 then
				listePerso[i] = me 
				numPersoSelectionne = i 
			--else
			--	numPersoSelectionne = 1
			--	alt = listePerso[1]
			--end
		end

		local function OnLDBClick(frame, btn)
			if btn == "LeftButton" then
				numPersoSelectionne = numPersoSelectionne + 1
				if numPersoSelectionne > #listePerso then numPersoSelectionne = 1 end
				alt = listePerso[numPersoSelectionne]
			else
				InterfaceOptionsFrame_OpenToCategory("AltManager")
			end
		end
	
		local function OnLDBShowTooltip(tt)
			tt:AddLine("AltManager")
			tt:AddLine(alt)
			
			if self:ShouldIShow(alt, "raids") then
				tt:AddLine(" ")
				tt:AddLine("|cFFFFFFFF"..L.RaidIDs.."|r")
				for k,v in pairs(taskList.raids) do
					if self:Handle(alt,v) and self:IsCapableOf(alt,v) then tt:AddLine(self:ttformat(L[v],self:GetStatus(alt,v))) end
				end
			end
			
			if self:ShouldIShow(alt, "jobs") then
				tt:AddLine(" ")
				tt:AddLine("|cFFFFFFFF"..L.JobQuests.."|r")
				for k,v in pairs(taskList.jobs) do
					if self:Handle(alt,v) and self:IsCapableOf(alt,v) then tt:AddLine(self:ttformat(L[v],self:GetStatus(alt,v))) end
				end
			end
			
			if self:ShouldIShow(alt, "misc") then
				tt:AddLine(" ")
				tt:AddLine("|cFFFFFFFF"..L.Misc.."|r")
				for k,v in pairs(taskList.misc) do
					if self:Handle(alt,v) and self:IsCapableOf(alt,v) then tt:AddLine(self:ttformat(L[v],self:GetStatus(alt,v))) end
				end
			end
			
			if self:ShouldIShow(alt, "pQuest") then
				tt:AddLine(" ")
				tt:AddLine("|cFFFFFFFF"..L.PersonalQuest.."|r")
				for k,v in pairs(self.db.global[alt].pQuest) do
					 tt:AddLine(self:ttformat(v,self:GetStatus(alt,k)))
				end
			end
		end
		
		local AltManagerLauncher = LDB:NewDataObject("AltManager", {
			type = "data source",
			div = "launcher",
			icon = "Interface\\Icons\\SPELL_nature_invisibilty",
			OnClick = function(clickedframe, button)
				GameTooltip:ClearLines()
				OnLDBClick(clickedframe, button)
				OnLDBShowTooltip(GameTooltip)
				GameTooltip:Show()
			end,
			OnTooltipShow = function(tt)
				OnLDBShowTooltip(tt)
			end
		})
		
		if LDBIcon then
			LDBIcon:Register("AltManager", AltManagerLauncher, self.db.profile.minimap)
		end
	end
end

function AltManager:ShouldIShow(alt, task)
	if task == "pQuest" then
		if (self.db.global[alt][task]) then
			for k,v in pairs(self.db.global[alt][task]) do
				return true
			end
		end
	else
		for k,v in pairs(taskList[task]) do
			if self:IsCapableOf(alt, v) and self:Handle(alt, v) then return true end
		end
	end
	return false
end

function AltManager:AnythingOnTooltip(alt)
	for k,v in pairs({"raids","jobs", "misc", "pQuest"}) do
		if self:ShouldIShow(alt, v) then return true end
	end
	return false
end